"""
API MAPPING FOR Zendesk API V2
"""

mapping_table = {

    # Tickets
    'list_all_tickets': {
        'path': '/tickets.json',
        'method': 'GET',
    },
    'list_recent_tickets': {
        'path': '/tickets/recent.json',
        'method': 'GET',
    },
    'show_ticket': {
        'path': '/tickets/{{ticket_id}}.json',
        'method': 'GET',
    },
    'show_many_tickets': {
        'path': '/tickets/show_many.json',
        'valid_params': ['ids'],
        'method': 'POST'
    },
    'create_ticket': {
        'path': '/tickets.json',
        'method': 'POST',
    },
    'update_ticket': {
        'path': '/tickets/{{ticket_id}}.json',
        'method': 'PUT',
    },
    'update_many_tickets': {
        'path': '/tickets/update_many.json',
        'valid_params': ['ids'],
        'method': 'PUT',
    },
    'delete_ticket': {
        'path': '/tickets/{{ticket_id}}.json',
        'method': 'DELETE',
    },
    'delete_many_tickets': {
        'path': '/tickets/destroy_many.json',
        'valid_params': ['ids'],
        'method': 'DELETE',
    },
    'list_ticket_collaborators': {
        'path': '/tickets/{{ticket_id}}/collaborators.json',
        'method': 'GET',
    },
    'list_ticket_incidents': {
        'path': '/tickets/{{ticket_id}}/incidents.json',
        'method': 'GET',
    },

    # Ticket Audits
    'list_audits': {
        'path': '/tickets/{{ticket_id}}/audits.json',
        'method': 'GET',
    },
    'mark_audit_as_trusted': {
        'path': '/tickets/{{ticket_id}}/audits/{{audit_id}}/trust.json',
        'method': 'PUT',
    },

    # Ticket Metrics
    'list_metrics': {
        'path': '/tickets/{{ticket_id}}/metrics.json',
        'method': 'GET',
    },

    # Ticket Comments
    'list_comments': {
        'path': '/tickets/{{ticket_id}}/comments.json',
        'method': 'GET',
    },

    # Incremental Tickets
    'export_incremental_tickets': {
        'path': '/exports/tickets.json',
        'valid_params': ['start_time'],
        'method': 'GET',
    },

    # Ticket Fields
    'list_ticket_fields': {
        'path': '/ticket_fields.json',
        'method': 'GET',
    },
    'show_ticket_field': {
        'path': '/ticket_fields/{{ticket_field_id}}.json',
        'method': 'GET',
    },
    'create_ticket_field': {
        'path': '/ticket_fields.json',
        'method': 'POST',
    },
    'update_ticket_field': {
        'path': '/ticket_fields/{{ticket_field_id}}.json',
        'method': 'PUT',
    },
    'delete_ticket_field': {
        'path': '/ticket_fields/{{ticket_field_id}}.json',
        'method': 'DELETE',
    },

    # Views
    'list_views': {
        'path': '/views.json',
        'method': 'GET',
    },
    'list_active_views': {
        'path': '/views/active.json',
        'method': 'GET',
    },
    'get_view': {
        'path': '/views/{{view_id}}.json',
        'method': 'GET',
    },
    'get_view_tickets': {
        'path': '/views/{{view_id}}/tickets.json',
        'method': 'GET',
    },
    'execute_view': {
        'path': '/views/{{view_id}}/execute.json',
        'method': 'GET',
    },
    'preview_view': {
        'path': '/views/preview.json',
        'method': 'POST',
    },
    'count_many_views': {
        'path': '/views/count_many.json',
        'valid_params': ('ids',),
        'method': 'GET',
    },
    'count_view': {
        'path': '/views/{{view_id}}/count.json',
        'method': 'GET',
    },
    'list_tickets_in_view': {
        'path':'/views/{{view_id}}/tickets.json',
        'method': 'GET',
    },

    # Users
    'list_users': {
        'path': '/users.json',
        'valid_params': ('role','page'),
        'method': 'GET',
    },
    'list_users_in_group': {
        'path': '/groups/{{group_id}}/users.json',
        'valid_params': ('role','page'),
        'method': 'GET',
    },
    'list_users_in_organization': {
        'path': '/organizations/{{organization_id}}/users.json',
        'valid_params': ('role','page'),
        'method': 'GET',
    },
    'show_user': {
        'path': '/users/{{user_id}}.json',
        'method': 'GET',
    },
    'show_many_users': {
        'path': '/users/show_many.json',
        'valid_params': ['ids'],
        'method': 'POST',
    },
    'create_user': {
        'path': '/users.json',
        'method': 'POST',
    },
    'create_many_users': {
        'path': '/users/create_many.json',
        'method': 'POST',
    },
    'update_user': {
        'path': '/users/{{user_id}}.json',
        'method': 'PUT',
    },
    'delete_user': {
        'path': '/users/{{user_id}}.json',
        'method': 'DELETE',
    },
    'search_user': {
        'path': '/users/search.json',
        'valid_params': ['query', 'external_id'],
        'method': 'GET',
    },
    'show_me': {
        'path': '/users/me.json',
        'method': 'GET',
    },
    'list_user_requests': {
        'path': '/users/{{user_id}}/tickets/requested.json',
        'method': 'GET',
    },
    'list_user_ccd': {
        'path': '/users/{{user_id}}/tickets/ccd.json',
        'method': 'GET',
    },

    # Requests
    'list_requests': {
        'path': '/requests.json',
        'method': 'GET',
    },
    'list_open_requests': {
        'path': '/requests/open.json',
        'method': 'GET',
    },
    'list_solved_requests': {
        'path': '/requests/solved.json',
        'method': 'GET',
    },
    'list_ccd_requests': {
        'path': '/requests/ccd.json',
        'method': 'GET',
    },
    'list_requests_for_user': {
        'path': '/users/{{user_id}}/requests.json',
        'method': 'GET',
    },
    'show_request': {
        'path': '/requests/{{request_id}}.json',
        'method': 'GET',
    },
    'create_request': {
        'path': '/requests.json',
        'method': 'POST',
    },
    'update_request': {
        'path': '/requests/{{request_id}}.json',
        'method': 'PUT',
    },
    'list_comments': {
        'path': '/requests/{{request_id}}/comments.json',
        'method': 'GET',
    },
    'show_comment': {
        'path': '/requests/{{request_id}}/comments/{{comment_id}}.json',
        'method': 'GET',
    },

    # User Identities
    'list_user_identities': {
        'path': '/users/{{user_id}}/identities.json',
        'method': 'GET',
    },
    'show_user_identity': {
        'path': '/users/{{user_id}}/identities/{{identity_id}}.json',
        'method': 'GET',
    },
    'create_user_identity': {
        'path': '/users/{{user_id}}/identities.json',
        'method': 'POST',
    },
    'update_user_identity': {
        'path': '/users/{{user_id}}/identities/{{identity_id}}.json',
        'valid_params': ['identityverified'],
        'method': 'PUT',
    },
    'make_user_identity_primary': {
        'path': '/users/{{user_id}}/identities/{{identity_id}}/make_primary.json',
        'method': 'PUT',
    },
    'verify_user_identity': {
        'path': '/users/{{user_id}}/identities/{{identity_id}}/verify.json',
        'method': 'PUT',
    },
    'request_user_identity_verification': {
        'path': '/users/{{user_id}}/identities/{{identity_id}}/request_verification.json',
        'method': 'PUT',
    },
    'delete_user_identity': {
        'path': '/users/{{user_id}}/identities/{{identity_id}}.json',
        'method': 'DELETE',
    },

    # Groups
    'list_groups': {
        'path': '/groups.json',
        'method': 'GET',
    },
    'list_assignable_groups': {
        'path': '/groups/assignable.json',
        'method': 'GET',
    },
    'show_group': {
        'path': '/groups/{{group_id}}.json',
        'method': 'GET',
    },
    'create_group': {
        'path': '/groups.json',
        'method': 'POST',
    },
    'update_group': {
        'path': '/groups/{{group_id}}.json',
        'method': 'PUT',
    },
    'delete_group': {
        'path': '/groups/{{group_id}}.json',
        'method': 'DELETE',
    },

    # Group Memberships
    'list_memberships': {
        'path': '/group_memberships.json',
        'method': 'GET',
    },
    'list_memberships_for_user': {
        'path': '/users/{{user_id}}/group_memberships.json',
        'method': 'GET',
    },
    'list_memberships_for_group': {
        'path': '/groups/{{group_id}}/group_memberships.json',
        'method': 'GET',
    },
    'list_assignable_memberships': {
        'path': '/group_memberships/assignable.json',
        'method': 'GET',
    },
    'list_assignable_memberships_for_group': {
        'path': '/groups/{{groups_id}}/group_memberships/assignable.json',
        'method': 'GET',
    },
    'show_membership': {
        'path': '/group_memberships/{{group_membership_id}}.json',
        'method': 'GET',
    },
    'show_membership_for_user': {
        'path': '/users/{{user_id}}/group_memberships/{{group_membership_id}}.json',
        'method': 'GET',
    },
    'create_membership': {
        'path': '/group_memberships.json',
        'method': 'POST',
    },
    'create_membership_for_user': {
        'path': '/users/{{user_id}}/group_memberships.json',
        'method': 'POST',
    },
    'delete_membership': {
        'path': '/group_memberships/{{group_membership_id}}.json',
        'method': 'DELETE',
    },
    'delete_membership_for_user': {
        'path': '/users/{{user_id}}/group_memberships/{{group_membership_id}}.json',
        'method': 'DELETE',
    },
    'set_default_membership_for_user': {
        'path': '/users/{{user_id}}/group_memberships/{{group_membership_id}}/make_default.json',
        'method': 'PUT',
    },

    # Custom Agent Rules
    'list_custom_roles': {
        'path': '/custom_roles.json',
        'method': 'GET',
    },

    # Organizations
    'list_organizations': {
        'path': '/organizations.json',
        'method': 'GET',
    },
    'list_organization_tickets': {
        'path': '/organizations/{{organization_id}}/tickets.json',
        'method': 'GET',
    },
    'autocomplete_organizations': {
        'path': '/organizations/autocomplete.json',
        'valid_params': ['name'],
        'method': 'POST',
    },
    'show_organization': {
        'path': '/organizations/{{organization_id}}.json',
        'method': 'GET',
    },
    'create_organization': {
        'path': '/organizations.json',
        'method': 'POST',
    },
    'update_organization': {
        'path': '/organizations/{{organization_id}}.json',
        'method': 'PUT',
    },
    'delete_organization': {
        'path': '/organizations/{{organization_id}}.json',
        'method': 'DELETE',
    },

    # Organization Fields
    'list_organization_fields': {
        'path': '/organization_fields.json',
        'method': 'GET',
    },
    'show_organization_field': {
        'path': '/organization_fields/{{organization_field_id}}.json',
        'method': 'GET',
    },
    'create_organization_field': {
        'path': '/organization_fields.json',
        'method': 'POST',
    },
    'update_organization_field': {
        'path': '/organization_fields/{{organization_field_id}}.json',
        'method': 'PUT',
    },
    'delete_organization_field': {
        'path': '/organization_fields/{{organization_field_id}}.json',
        'method': 'DELETE',
    },

    # Search
    'search': {
        'path': '/search.json',
        'valid_params': ['query', 'sort_by', 'sort_order'],
        'method': 'GET',
    },
    'anonymous_search': {
        'path': '/portal_search.json',
        'valid_params': ['query'],
        'method': 'GET',
    },

    # Tags
    'list_tags': {
        'path': '/tags.json',
        'method': 'GET',
    },

    # Forums
    'list_forums': {
        'path': '/forums.json',
        'method': 'GET',
    },
    'list_forums_category': {
        'path': '/categories/{{category_id}}/forums.json',
        'method': 'GET',
    },
    'show_forum': {
        'path': '/forums/{{forum_id}}.json',
        'method': 'GET',
    },
    'create_forum': {
        'path': '/forums.json',
        'method': 'POST',
    },
    'update_forum': {
        'path': '/forums/{{forum_id}}.json',
        'method': 'PUT',
    },
    'delete_forum': {
        'path': '/forums/{{forum_id}}.json',
        'method': 'DELETE',
    },

    # Forum Subscriptions
    'list_forum_subscriptions': {
        'path': '/forum_subscriptions.json',
        'method': 'GET',
    },
    'list_forum_subscriptions_for_forum': {
        'path': '/forums/{{forum_id}}/subscriptions.json',
        'method': 'GET',
    },
    'show_forum_subscription': {
        'path': '/forum_subscriptions/{{forum_subscription_id}}.json',
        'method': 'GET',
    },
    'create_forum_subscription': {
        'path': '/forum_subscriptions.json',
        'method': 'POST',
    },
    'delete_forum_subscription': {
        'path': '/forum_subscriptions/{{forum_subscription_id}}.json',
        'method': 'DELETE',
    },

    # Categories
    'list_categories': {
        'path': '/categories.json',
        'method': 'GET',
    },
    'show_category': {
        'path': '/category/{{category_id}}.json',
        'method': 'GET',
    },
    'create_category': {
        'path': '/categories.json',
        'method': 'GET',
    },
    'update_category': {
        'path': '/category/{{category_id}}.json',
        'method': 'PUT',
    },
    'delete_category': {
        'path': '/category/{{category_id}}.json',
        'method': 'DELETE',
    },

    # Topics
    'list_topics': {
        'path': '/topics.json',
        'method': 'GET',
    },
    'list_topics_for_forum': {
        'path': '/forums/{{forum_id}}/topics.json',
        'method': 'GET',
    },
    'list_topics_for_user': {
        'path': '/users/{{user_id}}/topics.json',
        'method': 'GET',
    },
    'show_topic': {
        'path': '/topics/{{topic_id}}.json',
        'method': 'GET',
    },
    'show_many_topics': {
        'path': '/topics/show_many.json',
        'valid_params': ['ids'],
        'method': 'GET',
    },
    'create_topics': {
        'path': '/topics.json',
        'method': 'POST',
    },
    'update_topic': {
        'path': '/topics/{{topic_id}}.json',
        'method': 'PUT',
    },
    'delete_topic': {
        'path': '/topics/{{topic_id}}.json',
        'method': 'DELETE',
    },

    # Topic Comments
    'list_topic_comments': {
        'path': '/topics/{{topic_id}}/comments.json',
        'method': 'GET',
    },
    'list_topic_comments_for_user': {
        'path': '/users/{{user_id}}/topic_comments.json',
        'method': 'GET',
    },
    'show_topic_comment': {
        'path': '/topics/{{topic_id}}/comments/{{comment_id}}.json',
        'method': 'GET',
    },
    'show_topic_comment_for_user': {
        'path': '/users/{{user_id}}/top_comments/{{comment_id}}.json',
        'method': 'GET',
    },
    'create_topic_comments': {
        'path': '/topics/{{topic_id}}/comments.json',
        'method': 'POST',
    },
    'update_topic_comment': {
        'path': '/topics/{{topic_id}}/comments/{{comment_id}}.json',
        'method': 'PUT',
    },
    'delete_topic_comment': {
        'path': '/topics/{{topic_id}}/comments/{{comment_id}}.json',
        'method': 'DELETE',
    },

    # Topic Subscriptions
    'list_topic_subscriptions': {
        'path': '/topic_subscriptions.json',
        'method': 'GET',
    },
    'list_subscriptions_for_topic': {
        'path': '/topics/{{topic_id}}/subscriptions.json',
        'method': 'GET',
    },
    'show_topic_subscription': {
        'path': '/topic_subscriptions/{{topic_subscription_id}}.json',
        'method': 'GET',
    },
    'create_topic_subscription': {
        'path': '/topic_subscriptions.json',
        'method': 'POST',
    },
    'delete_topic_subscription': {
        'path': '/topic_subscriptions/{{topic_subscription_id}}.json',
        'method': 'DELETE',
    },

    # Topic Votes
    'list_topic_votes': {
        'path': '/topics/{{topic_id}}/votes.json',
        'method': 'GET',
    },
    'list_topic_votes_for_user': {
        'path': '/users/{{user_id}}/topic_votes.json',
        'method': 'GET',
    },
    'show_topic_vote': {
        'path': '/topics/{{topic_id}}/vote.json',
        'method': 'GET',
    },
    'create_topic_vote': {
        'path': '/topics/{{topic_id}}/vote.json',
        'method': 'POST',
    },
    'delete_topic_vote': {
        'path': '/topics/{{topic_id}}/vote.json',
        'method': 'DELETE',
    },

    # Account Settings
    'show_account_settings': {
        'path': '/account/settings.json',
        'method': 'GET',
    },

    # Activity Stream
    'list_activities': {
        'path': '/activities.json',
        'method': 'GET',
    },
    'show_activity': {
        'path': '/activities/{{activity_id}}.json',
        'method': 'GET',
    },

    # Attachments
    'upload_attachment': {
        'path': '/uploads.json',
        'method': 'POST',
        'valid_params': ['filename', 'token'],
    },

    # Job Statuses
    'show_job_status': {
        'path': '/job_statuses/{{job_id}}.json',
        'method': 'GET',
    },

    # Locales
    'list_locales': {
        'path': '/locales.json',
        'method': 'GET',
    },
    'list_locales_for_agents': {
        'path': '/locales/agent.json',
        'method': 'GET',
    },
    'show_locale': {
        'path': '/locales/{{locale_id}}.json',
        'method': 'GET',
    },
    'show_current_locale': {
        'path': '/locales/current.json',
        'method': 'GET',
    },

    # Macros
    'list_macros': {
        'path': '/macros.json',
        'method': 'GET',
    },
    'list_active_macros': {
        'path': '/macros/active.json',
        'method': 'GET',
    },
    'show_macro': {
        'path': '/macros/{{macro_id}}.json',
        'method': 'GET',
    },
    'apply_macro': {
        'path': '/macros/{{macro_id}}/apply.json',
        'method': 'GET',
    },
    'apply_macro_for_ticket': {
        'path': '/tickets/{{ticket_id}}/macros/{{macro_id}}/apply.json',
        'method': 'GET',
    },

    # List Satisfaction Ratings
    'list_satisfaction_ratings': {
        'path': '/satisfaction_ratings.json',
        'method': 'GET',
    },
    'list_received_satisfaction_ratings': {
        'path': '/satisfaction_ratings/received.json',
        'method': 'GET',
    },
    'show_satisfaction_rating': {
        'path': '/satisfaction_ratings/{{satisfaction_rating_id}}.json',
        'method': 'GET',
    },

    # Suspended Tickets
    'list_suspended_tickets': {
        'path': '/suspended_tickets.json',
        'method': 'GET',
    },
    'show_suspended_ticket': {
        'path': '/suspended_tickets/{{ticket_id}}.json',
        'method': 'GET',
    },
    'recover_suspended_ticket': {
        'path': '/suspended_tickets/{{ticket_id}}/recover.json',
        'method': 'PUT',
    },
    'recover_many_suspended_tickets': {
        'path': '/suspended_tickets/recover_many.json',
        'valid_params' : ['ids'],
        'method': 'PUT',
    },
    'delete_suspended_ticket': {
        'path': '/suspended_tickets/{{ticket_id}}.json',
        'method': 'DELETE',
    },
    'delete_many_suspended_tickets': {
        'path': '/suspended_tickets/destroy_many.json',
        'valid_params' : ['ids'],
        'method': 'DELETE',
    },
}

# Patch mapping table with correct HTTP Status expected
for method, api_map in mapping_table.iteritems():
    status = 200
    if (method.startswith('create_') and not method.startswith('create_many_')
        or method == 'upload_attachment'):
        status = 201
    api_map['status'] = status



"""
    Python Zendesk is wrapper for the Zendesk API. This library provides an
    easy and flexible way for developers to communicate with their Zendesk
    account in their application.

    Notes:
    API THROTTLE is not handled in this library:
        From Zendesk: The throttle will be applied once an API consumer
        reaches a certain threshold in terms of a maximum of requests per
        minute. Most clients will never hit this threshold, but those that do,
        will get met by a HTTP 503 response code and a text body of
        "Number of allowed API requests per minute exceeded"

    TICKETS AS AN END-USER is not handled in this library:
        There are a number of API calls for working with helpdesk tickets as
        if you were the person (end-user) who originally created them. At the
        moment, this library is primarily for admins and agents to interact

    FORUMS, ENTRIES, POSTS have not yet been implemented

"""

__author__ = "Max Gutman <max@eventbrite.com>"
__version__ = "1.1.1"

import re
import httplib2
import urllib
import base64
try:
    import simplejson as json
except:
    import json
from httplib import responses
from endpoints import mapping_table as mapping_table_v1
from endpoints_v2 import mapping_table as mapping_table_v2

V2_COLLECTION_PARAMS = [
    'page',
    'per_page',
    'sort_order',
]


class ZendeskError(Exception):
    def __init__(self, msg, code, response):
        self.msg = msg
        self.error_code = code
        self.response = response

    def __str__(self):
        return repr('%s: %s %s' % (self.error_code, self.msg, self.response))


class AuthenticationError(ZendeskError):
    pass


class RateLimitError(ZendeskError):
    pass

re_identifier = re.compile(r".*/(?P<identifier>\d+)\.(json|xml)")


def get_id_from_url(url):
    match = re_identifier.match(url)
    if match and match.group('identifier'):
        return match.group('identifier')


def clean_kwargs(kwargs):
    """Format the kwargs to conform to API"""

    for key, value in kwargs.iteritems():
        if hasattr(value, '__iter__') and type(value) is not str:
            kwargs[key] = ','.join(map(str, value))


class Zendesk(object):
    """ Python API Wrapper for Zendesk"""

    def __init__(self, zendesk_url, zendesk_username=None,
                 zendesk_password=None, use_api_token=False, headers=None,
                 client_args={}, api_version=1):
        """
        Instantiates an instance of Zendesk. Takes optional parameters for
        HTTP Basic Authentication

        Parameters:
        zendesk_url - https://company.zendesk.com (use http if not SSL enabled)
        zendesk_username - Specific to your Zendesk account (typically email)
        zendesk_password - Specific to your Zendesk account or your account's
            API token if use_api_token is True
        use_api_token - use api token for authentication instead of user's
            actual password
        headers - Pass headers in dict form. This will override default.
        client_args - Pass arguments to http client in dict form.
            {'cache': False, 'timeout': 2}
            or a common one is to disable SSL certficate validation
            {"disable_ssl_certificate_validation": True}
        """
        self.data = None

        # Set attributes necessary for API
        self.zendesk_url = zendesk_url.rstrip('/')
        self.zendesk_username = zendesk_username
        if use_api_token:
            self.zendesk_username += "/token"
        self.zendesk_password = zendesk_password

        # Set headers
        self.headers = headers
        if self.headers is None:
            self.headers = {
                'User-agent': 'Zendesk Python Library v%s' % __version__,
                'Content-Type': 'application/json'
            }

        # Set http client and authentication
        self.client = httplib2.Http(**client_args)
        if (self.zendesk_username is not None and
                self.zendesk_password is not None):
            self.client.add_credentials(
                self.zendesk_username,
                self.zendesk_password
            )

        self.api_version = api_version
        if self.api_version == 1:
            self.mapping_table = mapping_table_v1
        elif self.api_version == 2:
            self.mapping_table = mapping_table_v2
        else:
            raise ValueError("Unsupported Zendesk API Version: %d" %
                             (self.api_version,))

    def __getattr__(self, api_call):
        """
        Instead of writing out each API endpoint as a method here or
        binding the API endpoints at instance runttime, we can simply
        use an elegant Python technique to construct method execution on-
        demand. We simply provide a mapping table between Zendesk API calls
        and function names (with necessary parameters to replace
        embedded keywords on GET or json data on POST/PUT requests).

        __getattr__() is used as callback method implemented so that
        when an object tries to call a method which is not defined here,
        it looks to find a relationship in the the mapping table.  The
        table provides the structure of the API call and parameters passed
        in the method will populate missing data.

        TODO:
            Should probably url-encode GET query parameters on replacement
        """
        def call(self, **kwargs):
            """ """
            api_map = self.mapping_table[api_call]
            path = api_map['path']
            if self.api_version == 2:
                path = "/api/v2" + path

            method = api_map['method']
            status = api_map['status']
            valid_params = api_map.get('valid_params', ())
            # Body can be passed from data or in args
            body = kwargs.pop('data', None) or self.data
            # If requested, return all response information
            complete_response = kwargs.pop('complete_response', False)
            # Support specifying a mime-type other than application/json
            mime_type = kwargs.pop('mime_type', 'application/json')
            # Substitute mustache placeholders with data from keywords
            url = re.sub(
                '\{\{(?P<m>[a-zA-Z_]+)\}\}',
                # Optional pagination parameters will default to blank
                lambda m: "%s" % kwargs.pop(m.group(1), ''),
                self.zendesk_url + path
            )
            # Validate remaining kwargs against valid_params and add
            # params url encoded to url variable.
            for kw in kwargs:
                if (kw not in valid_params and
                        (self.api_version == 2 and
                         kw not in V2_COLLECTION_PARAMS)):
                    raise TypeError("%s() got an unexpected keyword argument "
                                    "'%s'" % (api_call, kw))
            else:
                clean_kwargs(kwargs)
                url += '?' + urllib.urlencode(kwargs)

            # the 'search' endpoint in an open Zendesk site doesn't return a
            # 401 to force authentication. Inject the credentials in the
            # headers to ensure we get the results we're looking for
            if re.match("^/search\..*", path):
                self.headers["Authorization"] = "Basic %s" % (
                    base64.b64encode(self.zendesk_username + ':' +
                                     self.zendesk_password))
            elif "Authorization" in self.headers:
                del(self.headers["Authorization"])

            if mime_type == "application/json":
                body = json.dumps(body)
                self.headers["Content-Type"] = "application/json"
            else:
                self.headers["Content-Type"] = mime_type

            # Make an http request (data replacements are finalized)
            response, content = \
                self.client.request(
                    url,
                    method,
                    body=body,
                    headers=self.headers
                )
            # Use a response handler to determine success/fail
            return self._response_handler(response, content, status,
                complete_response)

        # Missing method is also not defined in our mapping table
        if api_call not in self.mapping_table:
            raise AttributeError('Method "%s" Does Not Exist' % api_call)

        # Execute dynamic method and pass in keyword args as data to API call
        return call.__get__(self)

    @staticmethod
    def _response_handler(response, content, status, complete_response=False):
        """
        Handle response as callback

        If the response status is different from status defined in the
        mapping table, then we assume an error and raise proper exception

        Zendesk's response is sometimes the url of a newly created user/
        ticket/group/etc and they pass this through 'location'.  Otherwise,
        the body of 'content' has our response.
        """
        response_status = int(response.get('status', 0))

        if response_status != status:
            if response_status == 401:
                raise AuthenticationError(content, response_status, response)
            elif response_status == 429:
                raise RateLimitError(content, response_status, response)
            else:
                raise ZendeskError(content, response_status, response)

        if complete_response:
            if content.strip():
                content = json.loads(content)

            return {
                'response': response,
                'content': content,
                'status': status
            }

        # Deserialize json content if content exist. In some cases Zendesk
        # returns ' ' strings. Also return false non strings (0, [], (), {})
        if response.get('location'):
            return response.get('location')
        elif content.strip():
            return json.loads(content)
        else:
            return responses[response_status]



from zendesk import Zendesk

################################################################
## NEW CONNECTION CLIENT
################################################################
zendesk = Zendesk('https://yourcompany.zendesk.com', 'you@yourcompany.com', 'passwd')

# Are you getting an error such as...
# "SSL routines:SSL3_GET_SERVER_CERTIFICATE:certificate verify failed"?
zendesk = Zendesk('https://yourcompany.zendesk.com', 'you@yourcompany.com', 'passwd',
    client_args={
        "disable_ssl_certificate_validation": True
    }
)


################################################################
## TICKETS
################################################################

# List
zendesk.list_tickets(view_id=1) # Must have a view defined

# Create
new_ticket = {
    'ticket': { _
        'requester_name': 'Howard Schultz',
        'requester_email': 'howard@starbucks.com',
        'subject':'My Starbucks coffee is cold!',
        'description': 'please reheat my coffee',
        'set_tags': 'coffee drinks',
        'ticket_field_entries': [
            {
                'ticket_field_id': 1,
                'value': 'venti'
            },
            {
                'ticket_field_id': 2,
                'value': '$10'
            }
        ]
    }
}
ticket_url = zendesk.create_ticket(data=new_ticket)

# Need ticket ID?
from zendesk import get_id_from_url
ticket_id = get_id_from_url(ticket_url)

# Show
zendesk.show_ticket(ticket_id=ticket_id)

# Delete
zendesk.delete_ticket(ticket_id=ticket_id)


################################################################
## ORGANIZATIONS
################################################################

# List
zendesk.list_organizations()

# Create
new_org = {
    'organization': {
        'name': 'Starbucks Corp'
    }
}
org_url = zendesk.create_organization(data=new_org)
org_id = get_id_from_url(org_url)

# Show
zendesk.show_organization(organization_id=org_id)

# Delete
zendesk.delete_organization(organization_id=org_id)


################################################################
## USERS (AGENTS)
################################################################

# List
zendesk.list_users()

# Create
new_user = {
    'user': {
        'name': 'Howard Schultz',
        'email': 'howard@starbucks.com',
        'roles': 4,
    }
}
user_url = zendesk.create_user(data=new_user)
user_id = get_id_from_url(user_url)

# Show
zendesk.show_user(user_id=user_id)

# Delete
zendesk.delete_user(user_id=user_id)


################################################################
## GROUPS
################################################################

# List
zendesk.list_groups()

# Create
new_group = {
    'group': {
        'name': 'Starbucks Group',
        'agents': [
            {
                'agent': 123
            },
        ]
    }
}
group_url = zendesk.create_group(data=new_group)
group_id = get_id_from_url(group_url)

# Show
zendesk.show_group(group_id=group_id)

# Delete
zendesk.delete_group(group_id=group_id)


################################################################
## TAGS
################################################################

# List
zendesk.list_tags()

# Show
zendesk.list_assets(tag_id=123, asset_type='event') # event | entry


################################################################
## TICKET TYPES
################################################################
zendesk.list_ticket_fields()


################################################################
## SEARCH
################################################################

# http://www.zendesk.com/api/search
# make sure to url-encode the query
results = zendesk.search(query='ticket+sort:desc', page=1)


from setuptools import setup
import sys

extra = {}
if sys.version_info >= (3,):
    extra['use_2to3'] = True

setup(
	# Basic package information.
	name = 'zdesk',
	author = 'Brent Woodruff, Max Gutman, Stefan Tjarks',
	version = '1.2.0',
	author_email = 'brent@fprimex.com',
	packages = ['zdesk'],
	include_package_data = True,
	install_requires = ['httplib2', 'simplejson'],
	license='LICENSE.txt',
	url = 'https://github.com/fprimex/zdesk',
	keywords = 'zendesk api helpdesk',
	description = 'Python API Wrapper for Zendesk',
	classifiers = [
		'Development Status :: 5 - Production/Stable',
		'Intended Audience :: Developers',
		'License :: OSI Approved :: MIT License',
		'Topic :: Software Development :: Libraries :: Python Modules',
		'Topic :: Internet',
		'Programming Language :: Python :: 2',
		'Programming Language :: Python :: 3',
	],
    **extra
)


